function dy = Derivative_1D(y, method)
% 
% dy = Derivative_1D(y, method)
%
% This function computes 1D spatial derivatives for y
%
% h = 1 is assumed; to get correct scaling:
%                    dy/h      for the 1st order
%                    dy/h^2    for the 2nd order
%
% y ->   (N x K) matrix: N x-points and K curves
%
% method ->  '1central'   - 1st derivative (central)
%            '2simple'    - 2nd derivative (simple)
%            '2improved'  - 2nd derivative (improved)
%
% Valeri Makarov
% April 2009, release
%
if nargin < 2, 
    help Derivative_1D
    error('two arguments required'); 
end

[N, K] = size(y); % N number of x-points; K - number of curves
dy = zeros(N,K);
if N < 2, return; end 

switch method
    case '1central'
        dy(1,:) = y(2,:) - y(1,:);
        if N > 2,  
            dy(2:N-1,:) = 0.5*(y(3:N,:) - y(1:N-2,:));
        end
        dy(N,:) = y(N,:) - y(N-1,:);
    case '2simple'
        if N >= 3,
            dy(1,:) = y(1,:) - 2*y(2,:) + y(3,:);
            dy(2:N-1,:) = y(3:N,:) - 2*y(2:N-1,:) + y(1:N-2,:);
            dy(N,:) = y(N,:) - 2*y(N-1,:) + y(N-2,:);
        end
    case '2improved'
        if N >= 5,
            dy(2,:) = y(3,:) - 2*y(2,:) + y(1,:);
            dy(3:N-2,:) = 0.25*(y(5:N,:) - 2*y(3:N-2,:) + y(1:N-4,:));
            dy(N-1,:) = y(N,:) - 2*y(N-1,:) + y(N-2,:);
            
            dy(1,:) = 2*dy(2,:) - dy(3,:);
            dy(N,:) = 2*dy(N-1,:) - dy(N-2,:);
        elseif N >= 3,
            dy(1,:) = y(1,:) - 2*y(2,:) + y(3,:);
            dy(2:N-1,:) = y(3:N,:) - 2*y(2:N-1,:) + y(1:N-2,:);
            dy(N,:) = y(N,:) - 2*y(N-1,:) + y(N-2,:);
        end
    otherwise
        help Derivative_1D
        error('unknown method')
end

