function [d, NORM, p] = HilbertDistance(u, Metrics, KAPPA)
%
% [d, NORM, p] = HilbertDistance(u, Metrics)
%
% Hilbert pair-wise distance (for 1D functions)
%
% u -> (j Points, n curves) matrix
%
% Metrics -> 'cos', 'H0', 'H1', 'H2'
%
% d    - distances in the selected metrics (use squareform(d) to reperesent as a matrix
% NORM - norm for each element (curve)
% p    - inner product  
%
% dimensional constant KAPPA = 2
% 1D Integration uses the Trapezoidal Rule
%
%
% for details: Makarov, Makarova, Herreras, J Comp. Neurosci.
%
% vmakarov@mat.ucm.es
%

if nargin < 2, 
    help HilbertDistance
    d = []; NORM = []; p = [];
    return
end
if nargin < 3 || isempty(KAPPA), KAPPA = 2; end 

[nPts, n] = size(u);
p = zeros(1,n*(n-1)/2);
t2 = 0;

switch Metrics
    case 'cos'
        warning('off','stats:pdist:APIChanged'); % no annoing warning
        p = 1-pdist(u','cosine');
        warning('on','stats:pdist:APIChanged');
    case 'H0'
        NORM = sqrt(diag(SIP(u,u)))';
        u = u./(ones(nPts,1)*NORM); % normalize loadings ||u|| = 1
        for k = 1:n-1,
            t1 = t2 + 1;
            t2 = t1 + (n-k) - 1;
            p(t1:t2) = SIP(u(:,k), u(:,k+1:n));
        end
    case 'H1'
        du = KAPPA*Derivative_1D(u, '1central');
        NORM = sqrt(diag(SIP(u,u)) + diag(SIP(du,du)))';
        u = u./(ones(nPts,1)*NORM); % normalize loadings ||u|| = 1
        du = KAPPA*Derivative_1D(u, '1central');
        for k = 1:n-1,
            t1 = t2 + 1;
            t2 = t1 + (n-k) - 1;
            p(t1:t2) = SIP(u(:,k), u(:,k+1:n))+SIP(du(:,k), du(:,k+1:n));
        end
    case 'H2'
        du  = KAPPA*Derivative_1D(u, '1central');
        ddu = KAPPA*KAPPA*Derivative_1D(u, '2improved');
        ddu = ddu(2:end-1,:);
        NORM = sqrt(diag(SIP(u,u)) + diag(SIP(du,du)) + diag(SIP(ddu,ddu))*nPts/(nPts-2))';
        u = u./(ones(nPts,1)*NORM); % normalize loadings ||u|| = 1
        du  = KAPPA*Derivative_1D(u, '1central');
        ddu = KAPPA*KAPPA*Derivative_1D(u, '2improved');
        ddu = ddu(2:end-1,:);
        for k = 1:n-1,
            t1 = t2 + 1;
            t2 = t1 + (n-k) - 1;
            p(t1:t2) = SIP(u(:,k), u(:,k+1:n)) + SIP(du(:,k), du(:,k+1:n)) +...
                SIP(ddu(:,k), ddu(:,k+1:n))*nPts/(nPts-2);
        end
end

d = 1 - abs(p);
end

function P = SIP(u,v)
%
% simple inner product <u,v> = Integral(uv)
%
P = u'*v - 0.5*u([1,end],:)'*v([1,end],:);
end
