function [A, s, idIN] = MAKE_ICA(eeg, varargin)
%
% [A, s, idIN] = MAKE_ICA(eeg, varargin) 
%
% eeg  -> (nChannels x nMeasurments) input matrix
%
% A    -> mixing matrix (loadings)
% s    -> time activations
% idIN -> most powerful components (> 10% of total variance) 
% 
% ICA_TYPE       -> 'runica' - default algorithm; 'e_runica' - extended runica;
%                   'fastica', 'kdica'
% NDIM           -> 'auto' or number
%
% NDIM_THRESHOLD -> in the case of NDIM = 'auto' it defines the threshold
%                   (PCA variance to retain)
%
% UNEXPL_VAR_FLAG -> If this flag is set to 'yes' or 'no' then no question
%                    dialg appears in the case of unexplained variance and 
%                    the action is taken according with the flag
% 
%
% Default values of parameters:
%
% VERBOSE  = 'on';
% NDIM = 'auto';
% ICA_TYPE = 'runica';
% NDIM_THRESHOLD = 0.99;
% UNEXPL_VAR_FLAG = 'none';
%
% Example:
%
% [A, s] = MAKE_ICA(eeg);
%
% [A, s, idIN] = MAKE_ICA(eeg, 'NDIM', 5, 'verbose', 'off', 'ica_type', 'fastica');
%
%
% Oct. 4, 2010 
%
% Oct. 20, 2011: Added KDICA option
% July 3, 2012: Added UNEXPL_VAR_FLAG
%

%

if nargin < 1,
    help MAKE_ICA
    error('Problem with the number of arguments in ICA_1');
end


% Set default values
VERBOSE  = 'on';
NDIM = 'auto';
ICA_TYPE = 'runica';
NDIM_THRESHOLD = 0.99;
UNEXPL_VAR_FLAG = 'none';

% Set optional values
if (rem(length(varargin),2)==1)
    error('Optional parameters should always go by pairs');
else
    for k = 1:2:(length(varargin)-1)
        if ~ischar (varargin{k}),
            error ([num2str(k) ' parameter is unknown (must be a strings).']);
        end
        switch lower(varargin{k})
            case 'verbose'
                VERBOSE = lower(varargin{k+1});
            case 'ndim'
                NDIM = varargin{k+1};
            case 'ica_type'
                ICA_TYPE = varargin{k+1};
            case 'ndim_threshold'
                NDIM_THRESHOLD = varargin{k+1};
            case 'unexpl_var_flag'
                UNEXPL_VAR_FLAG = varargin{k+1};
            otherwise
                error(['unknown parameter: ' varargin{k}])
        end
    end
end


if NDIM_THRESHOLD > 1, NDIM_THRESHOLD = 1; end

% define NDIM
if ischar(NDIM),
    [dum1, dum2, D] = pca(eeg');
    NDIM = find(cumsum(D)/sum(D) >= NDIM_THRESHOLD,1);
end
NDIM = min(NDIM,rank(eeg));
NDIM = round(NDIM);
% if NDIM == 1,
%     A(:,1) = dum1(:,1);
%     s(1,:) = dum2(:,1);
%     idIN = 1; 
%     return;
% end


switch lower(ICA_TYPE)
    case {'runica','e_runica'}
        if strcmpi(ICA_TYPE,'runica')
            ExtFlag = 0;
        else
            ExtFlag = 2;
        end
        if NDIM == size(eeg,1),  % No PCA reduction
            [weights,sphere,compvars,bias,signs,lrates,s] = runica(eeg, 'verbose', VERBOSE,...
                'maxsteps',1500, 'extended', ExtFlag);
        else  %    'PCA reduction'
            [weights,sphere,compvars,bias,signs,lrates,s] = runica(eeg, 'verbose', VERBOSE,...
                'pca', NDIM,'maxsteps',1500, 'extended', ExtFlag);
        end
        A = pinv(weights*sphere);
    case 'fastica'
        if NDIM == size(eeg,1),  % No PCA reduction
            [s, A, W] = fastica(eeg, 'finetune', 'pow3', 'verbose',  VERBOSE);
        else  %    'PCA reduction'
            [s, A, W] = fastica(eeg, 'lastEig', NDIM, 'finetune', 'pow3','verbose',  VERBOSE);
        end
    case 'kdica'
%         [A, s] = MakeFastKICA(eeg, NDIM);
        [A, s] = MakeKDICA(eeg, NDIM);
%          [A, s] = MakeERICA(eeg, NDIM);
    otherwise
        error([ICA_TYPE ' is not supported ica algorithm.']);
end

if ~isreal(s),
    errordlg('The ICA output is complex. The problem is badly posed. Consider adding noise.');
    s = 0*s(1,:);
    A = ones(size(eeg,1),1);
end

nCh = size(eeg,1);
y = eeg - A*s;
UnExpl = sum(y(:).^2)/sum(eeg(:).^2);
if UnExpl > 1 - NDIM_THRESHOLD,
    if strcmpi(UNEXPL_VAR_FLAG,'none')
        ButtonName = questdlg(['NDIM = ' num2str(NDIM) '. Unexplained variance = ' num2str(UnExpl*100,2)...
            '%. Extract ' num2str(NDIM+1) '-th generator?'], 'What to do next?', 'yes', 'no', 'yes');
    else
        ButtonName = UNEXPL_VAR_FLAG;
    end
    if strcmpi(ButtonName,'yes')
        V = zeros(nCh);
        for k = 1:nCh,
            for j = k:nCh,
                V(k,j) = mean(y(k,:).*y(j,:));
                V(j,k) = V(k,j);
            end
        end
        
        COEFF = pcacov(V);
        a1 = COEFF(:,1);
        
        s1 = mean(y.*repmat(a1,1,size(y,2)))/mean(a1.^2);
        A = [A a1];
        s = [s;s1];
    end
end

% indexes of "good" components
C = sum(A.^2).*mean(s.^2,2)';
Var = C/sum(C);
idIN = find(Var > 0.05); % at least 5% of the voltage variance

% polarity
for k = 1:size(A,2),
    [~, ii] = max(abs(A(:,k)));
    Vii = A(ii,k)*s(k,:);
    R = corrcoef(Vii,s(k,:));
    if R(1,2) < 0,
        % Polarity is inverted'
        s(k,:) = -s(k,:);
        A(:,k) = -A(:,k);
    end        
end 

% Normalize by max(A)
for k = 1:size(A,2)
    am = max(A(:,k));
    A(:,k) = A(:,k)/am;
    s(k,:) = s(k,:)*am; 
end