function PlotEEG(Data, Fs, ChanTitles, Scale, Title, T0, Clr, LnWd, units)
%
% PlotEEG(Data, Fs, ChanTitles, Scale, Title, T0, Clr, LnWd, units)
%
% This function plots EEG or analogous data 
%
% INPUT:
%
% Data       -> (nChan x nObserv) matrix of data
% Fs         -> sampling rate (optional, default: 256Hz)
% ChanTitles -> Cell with channel titles or leading string (optional, default: 'El#')
% Scale      -> Y scale for drawing (optional, default 0.6*max)
% Title      -> Graph title (optional, default: 'Data')
% T0         -> Initial value on the time axis (optional, default: 0) 
% Clr        -> Drawing Color (optional, e.g. 'brg', default: 'k')
%               if the number of colors < number of signals, then 
%               colors are recicled.
% LnWd       -> LineWidth (optional, defails 1)
% units      -> units to show in the scale plot (e.g. 'mV') (optional,
%               default [])
%
% Valeri A. Makarov, vmakarov@mat.ucm.es
% Sept. 2006
% March, 2007 (small improvements)
% Sept., 2008 added colors for signals + LineWidth
% April, 2009 fixed problem when Scale = 0 
% Sept., 2010 added 'units' 
%

if nargin < 1, 
    disp('No arguments');
    help PlotEEG
    return;
end
[nChs, nPts] = size(Data);
if nPts == 1, Data = Data'; [nChs, nPts] = size(Data); end

if nChs > 32, 
    disp('Too many channels (try to transpose the data)');
    return
end
if nargin < 2 || isempty(Fs),
    Fs = 256;   % default sampling rate
end
if nargin < 3 || isempty(ChanTitles),
    ChanTitles = [];   % default channel titles
end
if nargin < 4 || isempty(Scale),
    Scale =0.6*(max(Data(:)) - min(Data(:))); % automatic Y-scale
end
if Scale == 0, Scale = 1; end
if nargin < 5 || isempty(Title), Title = 'Data'; end
if nargin < 6 || isempty(T0), T0 = 0; end
if nargin < 7 || isempty(Clr), Clr = 'k'; end
if nargin < 8 || isempty(LnWd), LnWd = 1; end
if nargin < 9 || isempty(units), units = []; end

t = T0 + (0:nPts-1)/Fs;
Ct = {};
CLR_LEN = length(Clr);
for k = 1:nChs,
    if strcmp(Clr(mod(k - 1, CLR_LEN) + 1),'a')
        plot(t, (nChs - k)*Scale + Data(k,:), 'color',[0.8 0.8 0.8],'LineWidth',LnWd);
    else
        plot(t, (nChs - k)*Scale + Data(k,:), Clr(mod(k - 1, CLR_LEN) + 1),'LineWidth',LnWd);
    end
    hold on
    if isempty(ChanTitles),
        Ct = [Ct; {['E' num2str(nChs - k + 1)]}];
    elseif iscell(ChanTitles)
        Ct = [Ct; ChanTitles(nChs - k + 1)];
    else
        Ct = [Ct; {[ChanTitles num2str(nChs - k + 1)]}];
    end
end
plot((t(1) + 0.95*(t(end)-t(1)))*[1 1],[0 -Scale],'r','LineWidth',3)
h = text((t(1) + 0.96*(t(end)-t(1))),-Scale/2,[num2str(Scale,3) units]);
set(h,'color','r','BackgroundColor','w')
hold off;
xlim(T0 + [0, nPts-1]/Fs);
ylim([-1 nChs]*Scale);
title(Title,'FontSize',14);
set(gca,'YTick', (0:1:nChs-1)*Scale);
set(gca,'YTickLabel',Ct);
set(gca,'FontSize',10);
xlabel('time (s)','FontSize',12)
