function varargout = PlotLoadings(A, mark, ID, SE, xLmts, clrs)
%
% [AA, xx] = PlotLoadings(A, mark, ID, SE, xLmts, clrs)
%
% Spline plot of loadings
% 
% A    - matrix of loadings (nchannel x nloadings)
%
% mark - (optional) symbols to use as a marker, e.g. 'o', 's+',...
%
% ID   - (optional) indexes of loadings to plot (defauls all)
%
% SE   - (optional) Standard errors (nchannel x nloadings) ads error bars
%                   to the plot
% xLmts - (optional) x-limits for ploting (default: from 1 to nchannel)
% clrs  - (optional) colors to use for curves 'bgrcmyk'
%
% output:
% AA, xx - spline used for ploting
%
% Valeri Makarov, vmakarov@mat.ucm.es
%
% Nov. 2008, release
% Feb. 2009, added optional error bar plot
% Jan. 2015, added optional colors, fixed marks
%

if nargin < 1,
    help PlotLoadings;
    return;
end
[nElc, nCrv] = size(A);

if nargin < 6 || isempty(clrs),
    clrs = 'bgrcmyk'; 
end
clrs = clrs(:);
while nCrv > length(clrs) 
    clrs = [clrs; clrs];
end
if nargin < 2 || isempty(mark),
    mark = 'osv*<>+';
end
mark = mark(:);
while nCrv > length(mark) 
    mark = [mark; mark];
end
mark = [clrs(1:nCrv) mark(1:nCrv)];

if nargin < 3 || isempty(ID), ID = 1:nCrv; end
if nargin < 4 || isempty(SE),
    ErrBarFlag = 0;
else
    ErrBarFlag = 1;
end
if nargin < 5 || isempty(xLmts),
    xLmts = [1, nElc];
end

if ErrBarFlag,
    for k = 1:length(ID),
        errorbar(1:nElc,A(:,ID(k)),SE(:,ID(k)),[mark(ID(k),1) '.'],'LineWidth',2);
        hold on
    end
end
xx = 1:0.25:nElc;
AA = zeros(length(xx), nCrv); 
for k = 1:length(ID),
    AA(:,ID(k)) = spline(1:nElc,A(:,ID(k)),xx);
    plot(xx,AA(:,ID(k)),[mark(ID(k),1) '-'],'LineWidth',2)
    hold on
end
plot([xx(1),xx(end)],[0 0],'k--')
for k = 1:length(ID),
    plot(1:nElc, A(:,ID(k)),mark(ID(k),:),'LineWidth',2,'MarkerFaceColor','w')
end
if diff(xLmts) == 0, xLmts = xLmts + [-0.5 0.5]; end
xlim(xLmts)
hold off
varargout = {AA, xx};
varargout = varargout(1:nargout);
