function PlotWT(E, Emax, Args, FlagWT, FlagLog, FlagColorBar)
%
% PlotWT(E, Emax, Args, FlagWT, FlagLog, FlagColorBar)
%
% This function plots results of the wavelet analysis
%
% E      -> matrix to plot 
% Emax   -> maximal value of E (default Emax = max(E))
% Args   -> is a structure with all parameters
% FlagWT -> 0 - plot significance level for coherence (default), 1 - for power spectrum
% FlagLog -> 0 - plot in log scale power spectrum (default) only if FlagWT = 1
% FlagColorBar -> 1 (default) plot color bar
%
% March, 2007: Log color scale for power spectra
% Feb. 2007, Valeri Makarov
% vmakarov@opt.ucm.es
%

if nargin < 3 || ~isstruct(Args),
    help PlotWT
    error('Arguments?');
end
if nargin < 4 || isempty(FlagWT), 
    FlagWT = 0; 
    FlagLog = 0;
end
if nargin < 5 || isempty(FlagLog), 
    FlagLog = 0; 
end
if nargin < 6 || isempty(FlagColorBar), 
    FlagColorBar = 1; 
end

if ~isreal(E), E = abs(E); end
if isempty(Emax), Emax = max(E(:)); end

b = Args.b;
sc = log2(Args.a);      % dummy vector of periods
coi = Args.COI;         % cone of influence
if FlagWT
    sig95 = Args.PPsig95;   % significnce level for the power spectrum
else
    sig95 = Args.sig95;     % significnce level for coherence
end
if FlagLog
    if length(Emax) == 1,
        Emin = max(min(E(:)),Emax(end)*0.0001); % max 4 orders of magnitude
    else
        Emin = Emax(1);
    end
    E(E == 0) = Emin;
    image(b, sc, 64*log2(E/Emin)/log2(Emax(end)/Emin));
else
    image(b, sc, 64*E/Emax(end));
end    
hold on
if ~isempty(coi)
    if Args.BlackWhite
        plot(b, log2(coi), 'k--', 'linewidth', 2);
    else
        plot(b, log2(coi), 'k--', 'linewidth', 2);
%         hcoi = fill([b(1) b b(end)],[sc(end) log2(coi) sc(end)],'w');
%         set(hcoi,'alphadatamapping','direct','facealpha',0.5)
    end
end
if ~isempty(sig95),
    sig95 = sig95(:)*ones(size(b));
    [c,h] = contour(b, sc, E./sig95, [1 1],'k');
    set(h,'linewidth',2)
end

tks = get(gca,'YTick')';
if tks(1) >= -2,
%    set(gca,'YTickLabel', num2str(2.^tks,3),'TickDir','out')
    set(gca,'YTickLabel', num2str(1./(2.^tks),3),'TickDir','out')
    ylabel('frequency (Hz)')
%    ylabel('period, (s)')
else
%     set(gca,'YTickLabel', num2str(1000*2.^tks,3),'TickDir','out')
    set(gca,'YTickLabel', num2str(1./(2.^tks),3),'TickDir','out')
    ylabel('frequency (Hz)')
end
if FlagColorBar
    if FlagLog,
        hb = colorbar;
        YTk = get(hb,'Ytick');
        PS = num2str(2.^(log2(Emax(end)/Emin)*linspace(0,1,length(YTk))')*Emin, 2);
        set(hb, 'YTickLabel', PS);
    else
        colorbar('Ytick', [1 32 64], 'YTickLabel', num2str([0;0.5;1]*Emax(end), 2));
    end
end
if Args.BlackWhite,
    map = colormap(gray);
    colormap(flipud(map));
end

