function [data, fs, ChRead, T, PreTrig, tWinds, units] = READ_SMR(fname, type, Chns, ttl, verbose, DWN)
%
% [data, fs, ChRead, T, PreTrig] = READ_SMR(fname, type, Chns, ttl, verbose, DWN)
%
% Reads SMR files (wavemarks, continuous potential, or event channels)
%
% Input:
%   fname --> File name to read from
%   type  --> (optional) Type of channels to read: 1 (default) - Continuous waveforms, 
%             2 - Events, 6 - Wavemarks
%   Chns  --> (optional) Channel numbers (vector) to read or 'all' to read all
%             if Chns = [] a dialog (GUI) will be open
%   ttl   --> (optional) Title for the open dialog if Chns = [].
%   verbose --> (optional) 'on' or 'off' print some messages (default 'on')
%   DWN   --   (optional) downsampling rate for continous channels (default 1)
%
% Output:
%   data  --> Data matrix (for Wavemarks: nChannels x nSpaikes x Spike
%             length)
%   fs    --> Sampling rate in Hz
%   ChRead--> Open channels 
%   T     --> Spikes time stamps (only for type = 6)
%   PreTrig-> Number of points before triger (only for type = 6)
%   tWinds -> For files with multiple windows [tstart tend]
%
% Valeri Makarov: vmakarov@mat.ucm.es
%
% Nov 2006, release
% May 2007, little improvements
% Feb 2009, added verbose 
% Abr 2010, added Downsample + reading of files with multiple epochs
% May 2010, added tWinds 
% Aug 2010, changed tWinds due to son library 2.32 (NOT TESTED)
% Sep 2010, added output arg 'units' (mV, V) for continuous channels
%

if nargin < 1, 
    help READ_SMR
    error('Argumnets');
end
if nargin < 2 || isempty(type), type = 1; end      % default continuous signals
if nargin < 3, Chns = []; end; %default all channels
if nargin < 4 || isempty(ttl), ttl = 'Select channel among'; end; % default title for channel selection
if nargin < 5 || isempty(verbose), verbose = 'on'; end; % default print messages
if nargin < 6 || isempty(DWN), DWN = 1; end; % default downsample rate

data = []; fs = []; ChRead = []; T = []; PreTrig = []; tWinds = []; units = [];

if ~exist(fname,'file'),
    if strcmpi(verbose,'on'),
        disp(['File ' fname ' does not exists!']);
    end
    return;
end

fid = fopen(fname);
Ch = SONChanList(fid);
idx = find([Ch.kind] == type); % get avilable channels
if type == 2, % get '+' and '-' event channels
    idx = sort([idx find([Ch.kind] == 3)]); 
end
if isempty(idx),
    if strcmpi(verbose,'on'), disp('No channels of this data type!'); end
    return;
end

ChRead = [Ch(idx).number];

if isnumeric(Chns) && ~isempty(Chns),  % Select channels to read
    ChRead = intersect(ChRead, Chns);
elseif isempty(Chns) && length(ChRead) > 1,
    answer = inputdlg([ttl ' [' num2str(ChRead) ']'],...
        'Select channel',1,{num2str(ChRead(1))});
    ChRead = intersect(ChRead, str2num(answer{1}));
end
tWinds = [];
for i = 1:length(ChRead)
    try
        [X, header] = SONGetChannel(fid, ChRead(i), 'scale');
        switch type
            case 1 % Continuous data
                X = X(1:DWN:end,:);
                data{i} = X(:)';
%                X = X(:,1:DWN:end)';
%                data{i} = X(:)';
                fs = 1e6/(DWN*header.sampleinterval);
                if isfield(header,'start')
                    tWinds = [header.start(:) header.stop(:)];  % changed Aug. 30 2010 son library 2.32 
%                    tWinds = [header.start(:) header.start(:)+size(X,1)/fs]; 
                end
                if isfield(header,'units') % Added Sept. 30 2010
                    units = header.units;
                else
                    units = 'V';
                end
            case 6 % Wavemarks
                data(i,:,:) = double(X.adc);
                fs = 1e6/header.sampleinterval;
                T(i,:,:) = X.timings;
                PreTrig = header.preTrig;
            case 2
                data{i} = X;
            otherwise
                if strcmpi(verbose,'on'), 
                    disp(['The data type ' num2str(type) ' is not supported yet'])
                end
        end
    catch
        data{i} = [];
        fs = [];
    end
end
fclose(fid);
