function W = WT_Of_SpikeTrain(T, ko, a, b, FlagSpeedUp)
%
% W = WT_Of_SpikeTrain(T, ko, a, b, FlagSpeedUp)
% This function performs continuous wavelet transform of a spike trian.
%
% For details see Eq. (1)-(5) in 
%             Castellanos et al J. Neurophysiol 98: 2537-2549, (2007).
% Input:
%        T  - spike train (a vector of spike occurences)
%        ko - Morlet parameter
%        a  - vector of scales (periods)
%        b  - time grid
%        FlagSpeedUp - 0: precise but very slow;
%                      1: speed up calcs (precision decreases)
%                      2: even faster but less precise
%                      default: 0
%
% Output:
%        W  - WT (complex) matrix W(a,b) 
%
% Dec. 2007, Valeri Makarov: New flag: FlagSpeedUp
% Algorithm with limited precision, but faster. Affect long time scales
% WT power coefficient decay = 1/e^10 on the longest time scale
%
% March 2007, Valeri Makarov: speed up calculus
%
%
% Jan. 2007, Valeri Makarov: release
% vmakarov@mat.ucm.es
%

if nargin < 4 || ~isvector(T), 
    help WT_Of_SpikeTrain
    error('No arguments'); 
end
if nargin < 5 || isempty(FlagSpeedUp),
    FlagSpeedUp = 0;
end
a = a(:);
nB = length(b);
W = zeros(length(a),nB);

T = T(:);
nSp = length(T);


% some factors to speed up
A = 1./a';
Arg2 = 2*pi*T*A;
P1 = cos(Arg2) - 1i*sin(Arg2);
P2 = 2*pi*A;
P3 = -A.^2/(2*ko^2);
switch(FlagSpeedUp)
    case 0, % precise but very slow
        for k = 1:nB,
            Arg = (b(k) - T).^2;
            A2 = exp(Arg*P3);
            Arg1 = b(k)*P2;
            SS = ones(nSp,1)*(cos(Arg1) + 1i*sin(Arg1));
            W(:,k) = sum(P1.*SS.*A2,1);
        end
    case 1, % fast but not precise on the long time scales
        mP = 10*ko^2*a(end)^2; % wavelet power decreses up to e^10 on the longest scale
        parfor k = 1:nB,
            Arg = (b(k) - T).^2;
            idx = (Arg < mP);
            if sum(idx)
                A2 = exp(Arg(idx)*P3);
                Arg1 = b(k)*P2;
                SS = ones(sum(idx),1)*(cos(Arg1) + 1i*sin(Arg1));
                W(:,k) = sum(P1(idx,:).*SS.*A2,1);
            end
        end
    case 2, % even faster but less precise
        na = length(a);
        na2 = round(2*na/3);
        mP1 = 8*ko^2*a(na2)^2; % wavelet power decreses up to e^10 on the longest scale
        mP2 = 8*ko^2*a(na)^2; % wavelet power decreses up to e^10 on the longest scale
        P3lsc = P3(na2+1:na);
        P3ssc = P3(1:na2);
        P2lsc = P2(na2+1:na);
        P2ssc = P2(1:na2);
        P1lsc = P1(:,na2+1:na);
        P1ssc = P1(:,1:na2);
        for k = 1:nB,
            Arg = (b(k) - T).^2;
            idx = (Arg < mP2);  % long scales
            if sum(idx)
                A2 = exp(Arg(idx)*P3lsc);
                Arg1 = b(k)*P2lsc;
                SS = ones(sum(idx),1)*(cos(Arg1) + 1i*sin(Arg1));
                W(na2+1:na,k) = sum(P1lsc(idx,:).*SS.*A2,1);

                idx = (Arg < mP1); % short scales
                if sum(idx)
                    A2 = exp(Arg(idx)*P3ssc);
                    Arg1 = b(k)*P2ssc;
                    SS = ones(sum(idx),1)*(cos(Arg1) + 1i*sin(Arg1));
                    W(1:na2,k) = sum(P1ssc(idx,:).*SS.*A2,1);
                end
            end
        end
    otherwise
        error('Invalid value of the flag FlagSpeedUp')
end

W = W./repmat(sqrt(a),1,size(W,2));

% Firing rate of the spike train
%FR = sum(T >= b(1) & T <= b(end))/(b(end) - b(1));
% W = W/(pi^0.25 * sqrt(ko*FR));  % normalization to get P(noise) = 1